"""
Uses a rational RBF method and a modified partion of unity approach to 
solve Burgers' equation in an "amoeba" shaped domain.

u_t + u*u_x + u*u_y = nu*( u_{xx} + u_{yy} )

The initial condition and boundary conditions are taken from the exact
solution

u(x,y,t) = 1.0/( 1 + exp(( x + y - t + 1 )/(2*nu)) )

usage: run burgersRRBF2dMPU.py

NOTE: 9/11/2023. BLOWS UP AFTER A SHORT PERIOD OF TIME.  SOLUTION BECOMES
      INFINITE AND THE EIGENSOLVER FAILS.  COMPARE TO MATLAB VERSION IN 
      /papers/2017/ratRbf/c that works fine.

"""

from pylab import *
import rbfRational as rat
import rbfCentersLib as cl
import matplotlib.tri as tri
import AnimatePlot as ap
from rbfClassX import rbfx
import rbfMisc as rm
from iqx import *
import warnings                     # ignore warnings of ill-conditioning
warnings.filterwarnings('ignore')
close('all')

mu = 5e-15    # MDI regularization parameter
nu = 0.02     # Burgers' viscosity parameter
t, dt, finalT = 0, 0.005, 0.032
P = 1     # plot every Pth time step

def exact(x,y,t): return 1.0/( 1 + exp( (x + y - t + 1)/(2*nu) ) )

phi = iqx()

import rbfPU as PU
from W32 import *
phiC = W32()           # Wendland compactly supported RBF

xc, yc, bpi = cl.rbfCentersLib(1)
N, X = len(xc), list( zip(xc,yc) )

triang = tri.Triangulation(xc, yc)
mask = rm.longSideMask(xc,yc, triang.triangles, 0.135)
triang.set_mask(mask)

radius, x, y, st = PU.cover2d(xc,yc,7,0.75,20)
n = len(st)                                     # number of patches
w = PU.patchWeights(st,x,y,xc,yc,radius,phiC)

#sp = PU.patchShape(X,st,phi,mu=0,s0=0.5,ds=0.01)
sp = 7.75*ones((n,))

lL, lB, lBi, h1x, h1y, h2x, h2y = [], [], [], [], [], [], []
for k in range(n):
	st0 = st[k]
	r, rx, ry = rbfx.distanceMatrix2d(xc[st0],yc[st0])
	B, H1x, H1y, H2x, H2y = phi.rbf(r,sp[k]), phi.D1(r,sp[k],rx), phi.D1(r,sp[k],ry), phi.D2(r,sp[k],rx), phi.D2(r,sp[k],ry)
	lB.append(B)
	L, Bi = rat.rbfRationalCoreMultipleSetUp(B)
	lL.append( L )
	lBi.append( Bi) 
	h1x.append( H1x ) 
	h1y.append( H1y )
	h2x.append( H2x )
	h2y.append( H2y )
	
def F(u,t):
	fp = zeros((N,))
	u[bpi] = exact(xc[bpi],yc[bpi],t)
	for k in range(n):
		w0, st0 = w[k], st[k]
		ux, uy, uxx, uyy = rat.diff2dMultiple(u[st0],lB[k],lBi[k],lL[k],h1x[k],h1y[k],h2x[k],h2y[k],True)
		fp[st0] += w0*( -u[st0]*ux - u[st0]*uy + nu*( uxx + uyy ) )

	return fp
	
	
	
def makePlot():
	tricontour(triang, u, 15, linewidths=1.5, colors='k')
	tricontourf(triang, u, 15, cmap=plt.cm.jet) 


	
u = exact(xc,yc,0)
nt, t = 0, 0
while nt*dt < finalT:
	u = rm.rk4(u,t,dt,F)
	nt += 1
	t += dt
	u[bpi] = exact(xc[bpi],yc[bpi],t)
	if nt%P==0: 
		ap.animate(makePlot)
		print('t = {:1.4f}'.format(t))
	
print('error: {:1.2e}'.format( norm( u - exact(xc,yc,t))))
